######################################################################################
######################################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

# BRUGS program for state-space modelling of dynamic Biomass production model
# Application to the Namibian Hake fishery data
# Chapter  11 - section 11.1

######################################################################################
######################################################################################


rm(list = ls())

library(BRugs)
library(coda)
library(boa)
library(MASS)

# create repertory "results" in the wdbase directory
dir.create(path = c("results_shaefer_q100"), recursive = TRUE, showWarnings = TRUE)
dir.create(path = c("results_shaefer_q500"), recursive = TRUE, showWarnings = TRUE)
dir.create(path = c("results_fox"), recursive = TRUE, showWarnings = TRUE)

# load a function needed to save mcmc results
source("function.save.mcmc.R")

# MCMC sampling

n_burnin <- 10000
n_thin <- 10
n_iter <- 10000
n_chains <- 3
n_draws <- n_iter*n_chains



# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
# Run Schaefer with quotas=100
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------

# Create the data file containing the indicatrice for using Schaeer or Fox model

data_form <- bugsData(data=list("form.schaefer"=1, "form.fox"=0),fileName="model_form_temp.txt")

monitor <- c( "B", "P", "F", "D",
		"r", "K", "q", "sigma2p", "C_MSY", "B_MSY", "F_MSY", "risk", 
		"C_MSY_p", 
		"I_pred")

# Run OpenBUGS model

model <- "model_BiomProd_SchaeferFox.txt"
data <- "data_BiomProd_quotas100.txt"
inits1 <- "inits1_BiomProd.txt"
inits2 <- "inits2_BiomProd.txt"
inits3 <- "inits3_BiomProd.txt"

modelCheck(model)
modelData(data)
modelData("model_form_temp.txt")
modelCompile(numChains=n_chains)
modelInits(c(inits1,inits2,inits3))
modelGenInits()

modelUpdate(numUpdates = n_burnin, thin = 1, overRelax = FALSE)
samplesSet(monitor)
dicSet()
modelUpdate(numUpdates = n_iter, thin = n_thin, overRelax = FALSE)


# Save results in /results_shaefer_q100
# --------------------------------------------------------------------------

setwd("./results_shaefer_q100")

DIC <- dicStats()
name <- "DIC"
write.table(x=DIC, file = name, append = F, quote = F)
stats <- samplesStats("*")
name <- "stats"
write.table(x=stats, file = name, append = F, quote = F)
for(i in 1:length(monitor)) {save.mcmc(var=monitor[i],write=TRUE)}

setwd("../")


# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
# Draw figures
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------

setwd("./results_shaefer_q100")

B_q100 <- as.matrix(read.table(file = "B", header = T, sep = ""))
P <- as.matrix(read.table(file = "P", header = T, sep = ""))
D <- as.matrix(read.table(file = "D", header = T, sep = ""))
F <- as.matrix(read.table(file = "F", header = T, sep = ""))
I_pred <- as.matrix(read.table(file = "I_pred", header = T, sep = ""))

r <- as.vector(read.table(file = "r", header = T, sep = ""))
K <- as.vector(read.table(file = "K", header = T, sep = ""))
q <- as.vector(read.table(file = "q", header = T, sep = ""))
C_MSY <- as.vector(read.table(file = "C_MSY", header = T, sep = ""))
C_MSY_p <- as.vector(read.table(file = "C_MSY_p", header = T, sep = ""))

setwd("../")

data <- source("data_BiomProd_quotas100.txt")$value
n.obs <- data$n.obs
n.proj <- data$n.proj
n <- n.obs + n.proj
Iobs <- data$I

# --------------------------------------------------------------------------
# Boxplot of time series
# --------------------------------------------------------------------------

size.text <- 1.3
size.labels <- 1
box.size <- 0.5
col.obs <- "white"
# col.proj <- "grey"
col.proj <- "grey75"
# col2 <- "darkgrey"
col2 <- "grey30"
col <- c(rep(col.obs, times=n.obs),rep(col.proj, times=n.proj))
col2 <- rep(col2, times=n)

# ----------------------------------------------
# Biomass
# ----------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "PostBiomassSchaefer_q100.png"
png(filename = name_figure, height = 500*res, width = 650*res, res=72*res)

par(mfrow = c(1,1), bty="n", mar=c(5,5,1,1))

X <- B_q100
years <- 1964:(1964+n-1)

x.min = 1 ; x.max = n+1
y.min = 0 ; y.max = 10000
line.y = y.max*2
y.label = "Biomass"
x.label = "Years"
title = "Biomass (X 1000 tons)"

boxplot(	X[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex = box.size, col = col[1]) 
for (i in 2:n) {
boxplot(X[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex = box.size, col = col[i]) }

points(x=x.min:(x.max-1),y=rep(line.y,n),col="red",type="l",lty=2,lwd="1")
axis(side =1, at=1:n, labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 2, side = 2, cex = size.text)

par(def.par)

dev.off()



# ----------------------------------------------
# Depletion
# ----------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "PostDepletionSchaefer.png"
png(filename = name_figure, height = 500*res, width = 650*res, res=72*res)

par(mfrow = c(1,1), bty="n", mar=c(5,5,1,1))

X <- D
n <- dim(X)[2]
years <- 1964:(1964+n-1)

x.min = 1 ; x.max = n+1
y.min = 0 ; y.max = 1.5
line.y = y.max*2
y.label = "B/Bo"
x.label = "Years"
title = "Proportion of the virgin biomass (=B/Bo)"

boxplot(	X[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex = box.size, col = col[1]) 
for (i in 2:n) {
boxplot(X[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex = box.size, col = col[i]) }

points(x=x.min:(x.max-1),y=rep(line.y,n),col="red",type="l",lty=2,lwd="1")
axis(side =1, at=1:n, labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 2, side = 2, cex = size.text)

par(def.par)

dev.off()


# ----------------------------------------------
# Fishing mortality
# ----------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "PostFishingMortalitySchaefer.png"
png(filename = name_figure, height = 500*res, width = 650*res, res=72*res)

par(mfrow = c(1,1), bty="n", mar=c(5,5,1,1))

X <- F
n <- dim(X)[2]
years <- 1964:(1964+n-1)

x.min = 1 ; x.max = n+1
y.min = 0 ; y.max = 0.6
line.y = y.max*2
y.label = "Fishing mortality"
x.label = "Years"
title = "Fishing mortality (F=C/B)"

boxplot(	X[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex = box.size, col = col[1]) 
for (i in 2:n) {
boxplot(X[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex = box.size, col = col[i]) }

points(x=x.min:(x.max-1),y=rep(line.y,n),col="red",type="l",lty=2,lwd="1")
axis(side =1, at=1:n, labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 2.5, side = 2, cex = size.text)

par(def.par)

dev.off()


# ----------------------------------------------
# Abundance indices - predicted vs observed
# ----------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "FitAbundanceIndices.png"
png(filename = name_figure, height = 500*res, width = 650*res, res=72*res)

par(mfrow = c(1,1), bty="n", mar=c(5,5,1,1))

X <- I_pred
years <- 1964:(1964+n.obs-1)

x.min = 1 ; x.max = n.obs+1
y.min = 0 ; y.max = 2
line.y = Iobs[1:n.obs]
y.label = "Abundance indices"
x.label = "Years"
title = "Predicted vs observed Abundance Indices"

boxplot(	X[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex = box.size, col = col[1]) 
for (i in 2:n.obs) {
boxplot(X[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex = box.size, col = col[i]) }

points(x=x.min:(x.max-1),y=line.y,col="grey40",type="l",lty=1,lwd=2)
axis(side =1, at=1:n.obs, labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)

par(def.par)

dev.off()


# --------------------------------------------------------------------------
# Posterior estimates of parameters with correlations
# --------------------------------------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "PostParametersShaefer.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

x1 <- r
x2 <- K
x3 <- q
x4 <- C_MSY
x <- cbind(x1,x2,x3,x4)
labels <- c("r","K","q","MSY")

panel.dens <- function(x, ...)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(usr[1:2], 0, 1.5) )
    h <- density(x,adjust=1)
    h$y<-h$y/max(h$y)
    xlim=range(h$x)
    lines(h, col="black", lwd=2, ...)
}

## put (absolute) correlations on the upper panels,
## with size proportional to the correlations.

panel.cor <- function(x, y, digits=2, prefix="", cex.cor)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(0, 1, 0, 1))
    r <- (cor(x, y))
    txt <- format(c(r, 0.123456789), digits=digits)[1]
    txt <- paste(prefix, txt, sep="")
    if(missing(cex.cor)) cex <- 0.8/strwidth(txt)
    text(0.5, 0.5, txt, cex = 1.5)
}

par(pch='.', cex=0.5)

pairs( x,labels=labels, 
	 lower.panel=panel.smooth,
	 diag.panel=panel.dens, 
	 upper.panel=panel.cor,
	 cex.labels = 1.5, font.labels=1.5)

par(def.par)

dev.off()




# --------------------------------------------------------------------------
# Joint posterior distribution for (r,K)
# --------------------------------------------------------------------------


def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "Jointposterior_RK_Shaefer.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

x <- r$r
y <- K$K

indice <- which(y<=quantile(y, prob = 0.975))
x <- x[indice]
y <- y[indice]

xrange <- range(x) ; nbreaks.x=100
yrange <- range(y) ; nbreaks.y=100

nb.points = 2000

xhist <- hist(x, breaks=seq(xrange[1],xrange[2],length.out=nbreaks.x), plot=FALSE)
yhist <- hist(y, breaks=seq(yrange[1],yrange[2],length.out=nbreaks.y), plot=FALSE)

nf <- layout(matrix(c(2,0,1,3),2,2,byrow=TRUE), c(3,1), c(1,3), TRUE)
layout.show(nf)

par(mar=c(5,5,1,1))
plot(x[1:nb.points], y[1:nb.points], xlim=xrange, ylim=yrange, xlab="r", ylab="K", pch=".", cex.lab=size.text)
dens2d <- kde2d(x=x, y=y, n = 100)
contour(dens2d , nlevels = 3, drawlabels=FALSE, col = "black", lw = 2, add = T) 

par(mar=c(0,5,3,1))
barplot(xhist$density, axes=FALSE, space=0, horiz=FALSE, col = "lightgrey")
mtext(text="Marginal pdf for r", side=3, line=1, cex = size.text)

par(mar=c(5,0,1,3))
barplot(yhist$density, axes=FALSE, space=0, horiz=TRUE, col = "lightgrey")
mtext(text="Marginal pdf for K", side=4, line = 1, las = 0, cex = size.text)

par(def.par)

dev.off()



# ----------------------------------------------------------------------------
# Posterior distribution of MSY
# ----------------------------------------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "PostMSYSchaefer.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(mar=c(5,5,1,1), bty="n")

densr <- density(r$r)
densK <- density(K$K)
densMSY <- density(C_MSY$C_MSY)
densMSYp <- density(C_MSY_p$C_MSY_p)

plot(densMSY ,ylab = "", xlab = "", xaxt="n", yaxt="n", main="", xlim = c(0,1000), col = "black", type = "l", lty = 1, lwd = 2)
points(densMSYp ,ylab = "", xlab = "", xaxt="n", yaxt="n", main="", xlim = c(0,1000), col = "black", type = "l", lty = 2, lwd = 2)

axis(side = 1, tick = T, cex.axis=1, las=1)
axis(side = 2, tick = T, lwd.ticks=0, labels=NA, cex.axis=1)
mtext(side=1, "MSY (x 1000 tons)", bty="n", line=3, cex = size.text)
mtext(side=2, "", bty="n", line=3, cex = size.text)

legend(	legend = c("Prior","Posterior"),
 		col = c("black","black"), 	
 		lty = c(2,1), lwd = c(2,2), x = "topright", cex = size.text, bty ="n") 

par(def.par)

dev.off()







# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
# Run Schaefer with quotas=500
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------

# Create the data file containing the indicatrice for using Schaeer or Fox model
bugsData(data=list("form.schaefer"=1, "form.fox"=0),fileName="model_form_temp.txt")

monitor <- c( "B", "P", "F", "D",
		"r", "K", "q", "sigma2p", "C_MSY", "B_MSY", "F_MSY", "risk", 
		"C_MSY_p")

model <- "model_BiomProd_SchaeferFox.txt"
data <- "data_BiomProd_quotas500.txt"
inits1 <- "inits1_BiomProd.txt"
inits2 <- "inits2_BiomProd.txt"
inits3 <- "inits3_BiomProd.txt"

modelCheck(model)
modelData(data)
modelData("model_form_temp.txt")	# Load the form parameter
modelCompile(numChains=n_chains)
modelInits(c(inits1,inits2,inits3))
modelGenInits()

modelUpdate(numUpdates = n_burnin, thin = 1, overRelax = FALSE)
samplesSet(monitor)
dicSet()
modelUpdate(numUpdates = n_iter, thin = n_thin, overRelax = FALSE)


# Store results in /results_shaefer_q500
# --------------------------------------

setwd("./results_shaefer_q500")
for(i in 1:length(monitor)) {save.mcmc(var=monitor[i],write=TRUE)}
setwd("../")


setwd("./results_shaefer_q500")
B_q500 <- as.matrix(read.table(file = "B", header = T, sep = ""))
setwd("../")



# --------------------------------------------------------------------------
# Compare biomass trajectories quotas = 100 vs quotas = 500
# --------------------------------------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "PostBiomassSchaefer.png"
png(filename = name_figure, height = 500*res, width = 650*res, res=72*res)

par(mfrow = c(1,1), mar=c(5,5,1,1), bty="n")

X <- B_q100
X2 <- B_q500

n <- dim(X)[2]
years <- 1964:(1964+n-1)

x.min = 1 ; x.max = n+1
y.min = 0 ; y.max = 10000
line.y = y.max*2
y.label = "Biomass (X 1000 tons)"
x.label = "Years"
title = "Biomass (X 1000 tons)"

boxplot(	X[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex = box.size, col = col[1]) 
for (i in 2:n) {
boxplot(X[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex = box.size, col = col[i]) }

for (i in (n.obs+1):n) {
boxplot(X2[,i], xaxt = "n", yaxt = "n", at = i+0.4, add = T, outpch = NA, boxwex = box.size, col = col2[i]) }

points(x=x.min:(x.max-1),y=rep(line.y,n),col="red",type="l",lty=2,lwd="1")
axis(side =1, at=1:n, labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)

par(def.par)

dev.off()






# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------
# Run Fox with quotas=100
# ----------------------------------------------------------------------------
# ----------------------------------------------------------------------------

# Create the data file containing the indicatrice for using Schaeer or Fox model
bugsData(data=list("form.schaefer"=0, "form.fox"=1),fileName="model_form_temp.txt")

monitor <- c( "B", "P", "F", "D",
		"r", "K", "q", "sigma2p", "C_MSY", "B_MSY", "F_MSY", "risk", 
		"C_MSY_p")

model <- "model_BiomProd_SchaeferFox.txt"
data <- "data_BiomProd_quotas100.txt"
inits1 <- "inits1_BiomProd.txt"
inits2 <- "inits2_BiomProd.txt"
inits3 <- "inits3_BiomProd.txt"

modelCheck(model)
modelData(data)
modelData("model_form_temp.txt")	# Load the form parameter
modelCompile(numChains=n_chains)
modelInits(c(inits1,inits2,inits3))
modelGenInits()

modelUpdate(numUpdates = n_burnin, thin = 1, overRelax = FALSE)
samplesSet(monitor)
dicSet()
modelUpdate(numUpdates = n_iter, thin = n_thin, overRelax = FALSE)


# Store results in /results_fox
# -----------------------------

setwd("./results_fox")
for(i in 1:length(monitor)) {save.mcmc(var=monitor[i],write=TRUE)}
setwd("../")


setwd("./results_fox")
B_Fox <- as.matrix(read.table(file = "B", header = T, sep = ""))
C_MSY_Fox <- as.vector(read.table(file = "C_MSY", header = T, sep = ""))
setwd("../")



# --------------------------------------------------------------------------
# Compare biomass trajectories Fox vs Schaefer
# --------------------------------------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "PostBiomassSchaeferFox.png"
png(filename = name_figure, height = 500*res, width = 650*res, res=72*res)

par(mfrow = c(1,1), mar=c(5,5,1,1), bty="n")

X <- B_q100
X2 <- B_Fox

n <- dim(X)[2]
years <- 1964:(1964+n-1)

x.min = 1 ; x.max = n+1
y.min = 0 ; y.max = 10000
line.y = y.max*2
y.label = "Biomass (X 1000 tons)"
x.label = "Years"
title = "Biomass (X 1000 tons)"

boxplot(	X[,1], 
		xaxt = "n", yaxt = "n", 
		xlim = c(x.min,x.max), ylim = c(y.min,y.max), 
		at = 1, outpch = NA, boxwex=box.size, col = col[1]) 
for (i in 2:n) {
boxplot(X[,i], xaxt = "n", yaxt = "n", at = i, add = T, outpch = NA, boxwex=box.size, col = col[i]) }

for (i in 1:n) {
boxplot(X2[,i], xaxt = "n", yaxt = "n", at = i+0.4, add = T, outpch = NA, boxwex=box.size, col = col2[i]) }

points(x=x.min:(x.max-1),y=rep(line.y,n),col="red",type="l",lty=2,lwd="1")
axis(side =1, at=1:n, labels = years, las=3, cex.axis=size.labels)
axis(side =2, cex.axis=size.labels)
mtext(x.label, line = 3, side = 1, cex = size.text)
mtext(y.label, line = 3, side = 2, cex = size.text)

par(def.par)

dev.off()


# --------------------------------------------------------------------------
# Compare CMSY Shaefer vs Fox
# --------------------------------------------------------------------------

def.par <- par(no.readonly = TRUE)

# windows()

res <- 6
name_figure <- "PostMSYSchaeferFox.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(mar=c(5,5,1,1), bty="n")

densMSY <- density(C_MSY$C_MSY)
densMSY_Fox <- density(C_MSY_Fox$C_MSY)

plot(densMSY_Fox ,ylab = "", xlab = "", xaxt="n", yaxt="n", main="", xlim = c(0,1000), col = "black", type = "l", lty = 2, lwd = 2)
points(densMSY ,ylab = "", xlab = "", xaxt="n", yaxt="n", main="", xlim = c(0,1000), col = "black", type = "l", lty = 1, lwd = 2)

axis(side = 1, tick = T, cex.axis=1, las=1)
axis(side = 2, tick = T, lwd.ticks=0, labels=NA, cex.axis=1)
mtext(side=1, "MSY", bty="n",line=3, cex = size.text)
mtext(side=2, "", bty="n", line=3, cex = size.text)

legend(	legend = c("Shaefer","Fox"),
 		col = c("black","black"), 	
 		lty = c(1,2), lwd = c(2,3), x = "topright", cex = size.text, bty ="n") 

par(def.par)

dev.off()

